@@ -6,8 +6,6 @@
  */
 
 use Drupal\Core\DependencyInjection\ContainerNotInitializedException;
-use Drupal\Core\Messenger\LegacyMessenger;
-use Drupal\Core\Url;
 use Symfony\Component\DependencyInjection\ContainerInterface;
 
 /**
@@ -82,7 +80,7 @@ class Drupal {
   /**
    * The current system version.
    */
-  const VERSION = '8.7.14';
+  const VERSION = '9.0.10';
 
   /**
    * Core API compatibility.
@@ -178,7 +176,7 @@ class Drupal {
    * @return string
    */
   public static function root() {
-    return static::getContainer()->get('app.root');
+    return static::getContainer()->getParameter('app.root');
   }
 
   /**
@@ -251,28 +249,19 @@ class Drupal {
   /**
    * Gets the current active user.
    *
+   * This method will return the \Drupal\Core\Session\AccountProxy object of the
+   * current user. You can use the \Drupal\user\Entity\User::load() method to
+   * load the full user entity object. For example:
+   * @code
+   *   $user = \Drupal\user\Entity\User::load(\Drupal::currentUser()->id());
+   * @endcode
+   *
    * @return \Drupal\Core\Session\AccountProxyInterface
    */
   public static function currentUser() {
     return static::getContainer()->get('current_user');
   }
 
-  /**
-   * Retrieves the entity manager service.
-   *
-   * @return \Drupal\Core\Entity\EntityManagerInterface
-   *   The entity manager service.
-   *
-   * @deprecated in Drupal 8.0.0 and will be removed before Drupal 9.0.0.
-   *   Use \Drupal::entityTypeManager() instead in most cases. If the needed
-   *   method is not on \Drupal\Core\Entity\EntityTypeManagerInterface, see the
-   *   deprecated \Drupal\Core\Entity\EntityManager to find the
-   *   correct interface or service.
-   */
-  public static function entityManager() {
-    return static::getContainer()->get('entity.manager');
-  }
-
   /**
    * Retrieves the entity type manager.
    *
@@ -365,12 +354,14 @@ class Drupal {
    *
    * This is the main entry point to the configuration API. Calling
    * @code \Drupal::config('book.admin') @endcode will return a configuration
-   * object in which the book module can store its administrative settings.
+   * object the Book module can use to read its administrative settings.
    *
    * @param string $name
-   *   The name of the configuration object to retrieve. The name corresponds to
-   *   a configuration file. For @code \Drupal::config('book.admin') @endcode, the config
-   *   object returned will contain the contents of book.admin configuration file.
+   *   The name of the configuration object to retrieve, which typically
+   *   corresponds to a configuration file. For
+   *   @code \Drupal::config('book.admin') @endcode, the configuration
+   *   object returned will contain the content of the book.admin
+   *   configuration file.
    *
    * @return \Drupal\Core\Config\ImmutableConfig
    *   An immutable configuration object.
@@ -542,45 +533,6 @@ class Drupal {
     return static::getContainer()->get('url_generator');
   }
 
-  /**
-   * Generates a URL string for a specific route based on the given parameters.
-   *
-   * This method is a convenience wrapper for generating URL strings for URLs
-   * that have Drupal routes (that is, most pages generated by Drupal) using
-   * the \Drupal\Core\Url object. See \Drupal\Core\Url::fromRoute() for
-   * detailed documentation. For non-routed local URIs relative to
-   * the base path (like robots.txt) use Url::fromUri()->toString() with the
-   * base: scheme.
-   *
-   * @param string $route_name
-   *   The name of the route.
-   * @param array $route_parameters
-   *   (optional) An associative array of parameter names and values.
-   * @param array $options
-   *   (optional) An associative array of additional options.
-   * @param bool $collect_bubbleable_metadata
-   *   (optional) Defaults to FALSE. When TRUE, both the generated URL and its
-   *   associated bubbleable metadata are returned.
-   *
-   * @return string|\Drupal\Core\GeneratedUrl
-   *   A string containing a URL to the given path.
-   *   When $collect_bubbleable_metadata is TRUE, a GeneratedUrl object is
-   *   returned, containing the generated URL plus bubbleable metadata.
-   *
-   * @see \Drupal\Core\Routing\UrlGeneratorInterface::generateFromRoute()
-   * @see \Drupal\Core\Url
-   * @see \Drupal\Core\Url::fromRoute()
-   * @see \Drupal\Core\Url::fromUri()
-   *
-   * @deprecated as of Drupal 8.0.x, will be removed before Drupal 9.0.0.
-   *   Instead create a \Drupal\Core\Url object directly, for example using
-   *   Url::fromRoute().
-   */
-  public static function url($route_name, $route_parameters = [], $options = [], $collect_bubbleable_metadata = FALSE) {
-    @trigger_error('Drupal::url() is deprecated as of Drupal 8.0.x, will be removed before Drupal 9.0.0. Instead create a \Drupal\Core\Url object directly, for example using Url::fromRoute()', E_USER_DEPRECATED);
-    return static::getContainer()->get('url_generator')->generateFromRoute($route_name, $route_parameters, $options, $collect_bubbleable_metadata);
-  }
-
   /**
    * Returns the link generator service.
    *
@@ -590,35 +542,6 @@ class Drupal {
     return static::getContainer()->get('link_generator');
   }
 
-  /**
-   * Renders a link with a given link text and Url object.
-   *
-   * This method is a convenience wrapper for the link generator service's
-   * generate() method.
-   *
-   * @param string $text
-   *   The link text for the anchor tag.
-   * @param \Drupal\Core\Url $url
-   *   The URL object used for the link.
-   *
-   * @return \Drupal\Core\GeneratedLink
-   *   A GeneratedLink object containing a link to the given route and
-   *   parameters and bubbleable metadata.
-   *
-   * @see \Drupal\Core\Utility\LinkGeneratorInterface::generate()
-   * @see \Drupal\Core\Url
-   *
-   * @deprecated in Drupal 8.0.0 and will be removed before Drupal 9.0.0.
-   *   Use \Drupal\Core\Link instead.
-   *   Example:
-   *   @code
-   *     $link = Link::fromTextAndUrl($text, $url);
-   *   @endcode
-   */
-  public static function l($text, Url $url) {
-    return static::getContainer()->get('link_generator')->generate($text, $url);
-  }
-
   /**
    * Returns the string translation service.
    *
@@ -775,9 +698,7 @@ class Drupal {
    *   The messenger.
    */
   public static function messenger() {
-    // @todo Replace with service once LegacyMessenger is removed in 9.0.0.
-    // @see https://www.drupal.org/node/2928994
-    return new LegacyMessenger();
+    return static::getContainer()->get('messenger');
   }
 
 }
