@@ -20,6 +20,11 @@ class CommentStatusFieldAccessTest extends BrowserTestBase {
    */
   public $profile = 'testing';
 
+  /**
+   * {@inheritdoc}
+   */
+  protected $defaultTheme = 'stark';
+
   /**
    * Comment admin.
    *
@@ -37,7 +42,7 @@ class CommentStatusFieldAccessTest extends BrowserTestBase {
   /**
    * {@inheritdoc}
    */
-  public static $modules = [
+  protected static $modules = [
     'node',
     'comment',
     'user',
@@ -48,7 +53,7 @@ class CommentStatusFieldAccessTest extends BrowserTestBase {
   /**
    * {@inheritdoc}
    */
-  protected function setUp() {
+  protected function setUp(): void {
     parent::setUp();
     $node_type = NodeType::create([
       'type' => 'article',
